import datetime
from pymongoose.mongo_types import Types, Schema

class Team (Schema):
	schema_name = "teams"

	id = None
	name = None
	competition = None
	division = None

	def __init__ (self, **kwargs):
		self.schema = {
			"name": {
				"type": Types.String,
				"required": True
			},
			"athletes": [{
				"type": Types.ObjectId,
				"ref": "athletes"
			}],
			"competition": {
				"type": Types.ObjectId,
				"ref": "competition",
				"required": True
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions",
				"required": True
			},
			"registration_date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			},
			"status": {
				"type": Types.Number,
				"default": 1
			},
			"role": {
				"type": Types.Number,
				"default": 1
			},
			"fitcard": {
				"type": Types.ObjectId,
				"ref": "teams.fitcards"
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Team: {self.id} - {self.name}"



from pymongoose.mongo_types import Types, Schema

scoring = {
	"0": None,
	"1": "Reps",
	"2": "Round + Reps",
	"3": "Time",
	"4": "Weight - lb",
	"5": "Weight - Kg",
	"6": "Distance - Imperial",
	"7": "Distance - Metric"
}

ranking = {
	"0": None,
	"1": "Lowest is Better",
	"2": "Highest is Better"
}

tie_breaker = {
	"0": None,
	"1": "Ties Stand",
	"2": "Reps",
	"3": "Round + Reps",
	"4": "Time",
	"5": "Weight - lb",
	"6": "Weight - Kg",
	"7": "Distance - Imperial",
	"8": "Distance - Metric"
}

class Workout (Schema):
	schema_name = "workouts"

	id = None
	name = None

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions"
			},
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": False
			},
			"scoring": {
				"type": Types.Number,
				"required": True
			},
			"ranking": {
				"type": Types.Number,
				"required": True
			},
			"tie_breaker": {
				"type": Types.Number,
				"required": False
			},
			"tie_ranking": {
				"type": Types.Number,
				"required": False
			},
			"tie_description": {
				"type": Types.String,
				"default": ""
			},
			"email": {
				"type": Types.Boolean,
				"default": False
			},
			"publish": {
				"type": Types.Boolean,
				"default": False
			},
			"publish_date": {
				"type": Types.Date,
				"required": True
			},
			"active_divisions": [{
				"type": Types.ObjectId,
				"ref": "divisions"
			}],
			"heats": [{
				"type": Types.ObjectId,
				"ref": "heats"
			}],
			"workout": {
				"type": Types.String,
				"required": False
			},
			"scores": {
				"type": Types.ObjectId,
				"ref": "scores"
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Workout: {self.id} - {self.name}"



from pymongoose.mongo_types import Types, Schema

class Division (Schema):
	schema_name = "divisions"

	id = None
	name = None
	description = None
	sold = None
	category = None
	n_athletes = None

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": True
			},
			
			"category": {
				"type": Types.Number,
				"default": 0,
				"required": True
			},
			
			"n_athletes": {
				"type": Types.Number,
				"required": True
			},
			"gender": {
				
				"n_persons_a": {
					"type": Types.Number,
					"default": 0
				},
				
				"n_persons_b": {
					"type": Types.Number,
					"default": 0
				},
				"free": {
					"type": Types.Number,
					"default": 0
				},
				
			},
			"minimum_age": {
				"type": Types.Number,
				"default": 18
			},
			# TODO:
			"benchmark": {
				# "name": {
				# 	"type": Types.String,
				# },
				# "b_type": {
				# 	"type": Types.Number
				# },
				# "ranking": {
				# 	"type": Types.Number,
				# },
				# "collect": {
				# 	"type": Types.Boolean,
				# 	"default": False
				# }
			},
			"waiver": {
				# "type": Types.Boolean,
				# "default": False
			},
			"color": {
				"type": Types.String,
				"default": "ffffffff"
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Division: {self.id} - {self.name}"



roles = None

def role_model_init (db):
	global roles
	roles = db["roles"]

class Role ():
	def __init__ (self):
		self.oid = None
		self.id = None
		self.name = None

		self.actions = []

	def __str__ (self):
		return f"Role: \n\t{self.id} \n\t{self.name}"

def role_parse (role_values):
	role = Role ()

	role.oid = role_values["_id"]
	role.id = str (role.oid)
	role.name = role_values["name"]

	role.actions = role_values["actions"]

	return role

def roles_get_all ():
	result = []

	all_roles = roles.find ({})
	if (all_roles is not None):
		for role in all_roles:
			result.append (role_parse (role))

	return result



import datetime
from pymongoose.mongo_types import Types, Schema

JUDGE_STATUS_NONE = 0
JUDGE_STATUS_ACTIVE = 1
JUDGE_STATUS_INACTIVE = 2

class Judge (Schema):
	schema_name = "judges"

	user = None
	status = JUDGE_STATUS_NONE

	def __init__ (self, **kwargs):
		self.schema = {
			"user": {
				"type": Types.ObjectId,
				"ref": "users",
				"required": True
			},

			"status": {
				"type": Types.Number,
				"default": JUDGE_STATUS_NONE
			},

			"registration_date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Judge: {self.id} - {self.user}"



FITLAND_VERSION = "0.2.2"
FITLAND_VERSION_NAME = "Version 0.2.2"
FITLAND_VERSION_DATE = "09/01/2022"
FITLAND_VERSION_TIME = "15:39 CST"
FITLAND_VERSION_AUTHOR = "Erick Salas"

version = {
	"id": FITLAND_VERSION,
	"name": FITLAND_VERSION_NAME,
	"date": FITLAND_VERSION_DATE,
	"time": FITLAND_VERSION_TIME,
	"author": FITLAND_VERSION_AUTHOR
}



from pymongoose.mongo_types import Types, Schema

class Permissions (Schema):
	schema_name = "permissions"

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"user": {
				"type": Types.ObjectId,
				"ref": "users",
				"required": True
			},
			"actions": [{
				"type": Types.String
			}]
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Permissions: {self.id}"



import datetime
from pymongoose.mongo_types import Types, Schema

class Athlete (Schema):
	schema_name = "athletes"

	id = None
	user = None
	username = None
	email = None
	competition = None
	division = None

	def __init__ (self, **kwargs):
		self.schema = {
			"user": {
				"type": Types.ObjectId,
				"ref": "users",
				"required": True
			},
			"username": {
				"type": Types.String,
				"required": True
			},
			"email": {
				"type": Types.String,
				"required": True
			},
			"manager": {
				"type": Types.String,
				"default": None
			},
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions",
				"required": True
			},
			"registration_date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			},
			"status": {
				"type": Types.Number,
				"default": 1
			},
			"role": {
				"type": Types.Number,
				"default": 1
			},
			"waiver": {
				"type": Types.String,
				"default": "no-waiver.pdf"
			},
			"benchmark_value": {
				"type": Types.Number,
				"default": -1
			},
			"tshirt_size": {
				"type": Types.Number,
				"default": 0
			},
			"fitcard": {
				"type": Types.ObjectId,
				"ref": "athletes.fitcards"
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Athlete: {self.id} - {self.registration_date}"



import datetime
from pymongoose.mongo_types import Types, Schema

class HeatSettings (Schema):
	schema_name = "heats.settings"

	def __init__ (self, **kwargs):
		self.schema = {
			"workout": {
				"type": Types.ObjectId,
				"ref": "workouts"
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions"
			},
			"initial_time": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			},
			"n_rails": {
				"type": Types.Number,
				"required": True
			},
			"workout_time_cap": {
				"type": Types.Number,
				"required": True
			},
			"time_between_heats": {
				"type": Types.Number,
				"required": True
			},
			"time_between_divisions": {
				"type": Types.Number,
				"required": True
			},
			"area": {
				"type": Types.ObjectId,
				"ref": "areas"
			},
			"reps" : {
				"type": Types.String,
				"required": False
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def _str_ (self):
		return f"Settings: {self.id}"



from .area import *
from .athlete_listeners import *
from .athlete_result import *
from .athlete_score import *
from .athlete import *
from .competition import *
from .division import *
from .heat import *
from .role import *
from .scores import *
from .settings import *
from .team_listeners import *
from .team_result import *
from .team_score import *
from .team import *
from .user import *
from .workout import *



import datetime
from pymongoose.mongo_types import Types, Schema

class Currency (Schema):
	schema_name = "currencies"

	name = None
	code = None
	description = None

	def __init__ (self, **kwargs):
		self.schema = {
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": True
			},
			"code": {
				"type": Types.String,
				"required": True
			},

			"registration_date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Currency: {self.id} - {self.code}"



import datetime
from pymongoose.mongo_types import Types, Schema

class User (Schema):
	schema_name = "users"

	id = None
	iat = None
	name = None
	surname = None
	username = None
	email = None
	password = None
	member_since = None
	birth_date = None
	box = None
	gender = None
	role = None

	def __init__ (self, **kwargs):
		self.schema = {
			"name": {
				"type": Types.String,
				"required": True
			},
			"surname": {
				"type": Types.String,
				"required": True
			},
			"username": {
				"type": Types.String,
				"required": True
			},
			"email": {
				"type": Types.String,
				"required": True
			},
			"password": {
				"type": Types.String,
				"required": True
			},
			"member_since": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			},
			"birth_date": {
				"type": Types.Date,
				"required": True
			},
			"role": {
				"type": Types.ObjectId,
				"ref": "roles"
			},
			"box": {
				"type": Types.String,
				"default": "None"
			},

			# 0 -> None
			# 1 -> Male
			# 2 -> Female
			# 3 -> Other
			"gender": {
				"type": Types.Number,
				"default": 0
			},

			"firebase": {
				"type": Types.String,
				"default": None
			},

			"logged_out": {
				"type": Types.Date,
				"default": None
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Name: {self.name}\nUsername: {self.username}\nEmail: {self.email}"



from pymongoose.mongo_types import Types, Schema

class Area (Schema):
	schema_name = "areas"

	def __init__ (self, **kwargs):
		self.schema = {
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": True
			},
			"n_rails": {
				"type": Types.Number,
				"required": True
			},
			"n_participants": {
				"type": Types.Number,
				"required": True
			},
			"status": {
				"type": Types.Number,
				"default": 1
			},
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions"
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f""



NOTIFICATION_TYPE_NONE = 0

# types of notification templates
NOTIFICATION_TYPE_GLOBAL = 1
NOTIFICATION_TYPE_WOD = 2
NOTIFICATION_TYPE_HEAT = 3
NOTIFICATION_TYPE_DIVISION = 4

# handled by events listeners
NOTIFICATION_TYPE_TEST = 5
NOTIFICATION_TYPE_SCORE = 6

# sorted in app
NOTIFICATION_TYPE_GENERAL = 1
NOTIFICATION_TYPE_TARGET = 2
NOTIFICATION_TYPE_FAVORITE = 3



import datetime
from pymongoose.mongo_types import Types, Schema

class Notification (Schema):
	schema_name = "notifications"

	title = None
	body = None
	n_type = None

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions",
			},
			"workout": {
				"type": Types.ObjectId,
				"ref": "workouts",
			},
			"heat": {
				"type": Types.ObjectId,
				"ref": "heats",
			},

			"title": {
				"type": Types.String,
				"required": True
			},
			"body": {
				"type": Types.String,
				"required": True
			},

			"exp_date": {
				"type": Types.Date,
				"default": None
			},
			"n_type": {
				"type": Types.Number,
				"default": 0
			},

			"date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Notification: {self.id}"



import datetime
from pymongoose.mongo_types import Types, Schema

STAFF_TYPE_NONE = 0
STAFF_TYPE_GENERAL = 1

STAFF_STATUS_NONE = 0
STAFF_STATUS_ACTIVE = 1
STAFF_STATUS_INACTIVE = 2

class Staff (Schema):
	schema_name = "staffs"

	user = None
	competition = None
	staff_type = STAFF_TYPE_NONE
	status = STAFF_STATUS_NONE

	def __init__ (self, **kwargs):
		self.schema = {
			"user": {
				"type": Types.ObjectId,
				"ref": "users",
				"required": True
			},
			
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},

			"staff_type": {
				"type": Types.Number,
				"default": STAFF_TYPE_NONE
			},

			"status": {
				"type": Types.Number,
				"default": STAFF_STATUS_NONE
			},

			"registration_date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Staff: {self.id} - {self.user} - {self.competition}"



from .lib import lib

from ctypes import c_void_p, c_char_p, c_int, c_uint, c_bool

# auth
BYND_AUTH_TYPE_NONE = 0
BYND_AUTH_TYPE_SINGLE = 1
BYND_AUTH_TYPE_MANAGEMENT = 2

bynd_auth_type_to_string = lib.bynd_auth_type_to_string
bynd_auth_type_to_string.argtypes = [c_int]
bynd_auth_type_to_string.restype = c_char_p

bynd_auth_delete = lib.bynd_auth_delete
bynd_auth_delete.argtypes = [c_void_p]

bynd_auth_get_type = lib.bynd_auth_get_type
bynd_auth_get_type.argtypes = [c_void_p]
bynd_auth_get_type.restype = c_int

bynd_auth_get_competition = lib.bynd_auth_get_competition
bynd_auth_get_competition.argtypes = [c_void_p]
bynd_auth_get_competition.restype = c_char_p

bynd_auth_get_action = lib.bynd_auth_get_action
bynd_auth_get_action.argtypes = [c_void_p]
bynd_auth_get_action.restype = c_char_p

bynd_auth_get_admin = lib.bynd_auth_get_admin
bynd_auth_get_admin.argtypes = [c_void_p]
bynd_auth_get_admin.restype = c_bool

bynd_auth_get_permissions = lib.bynd_auth_get_permissions
bynd_auth_get_permissions.argtypes = [c_void_p]
bynd_auth_get_permissions.restype = c_void_p

bynd_auth_permissions_iter_start = lib.bynd_auth_permissions_iter_start
bynd_auth_permissions_iter_start.argtypes = [c_void_p]
bynd_auth_permissions_iter_start.restype = c_bool

bynd_auth_permissions_iter_get_next = lib.bynd_auth_permissions_iter_get_next
bynd_auth_permissions_iter_get_next.argtypes = [c_void_p]
bynd_auth_permissions_iter_get_next.restype = c_void_p

bynd_auth_create = lib.bynd_auth_create
bynd_auth_create.argtypes = [c_int]
bynd_auth_create.restype = c_void_p

bynd_single_authentication = lib.bynd_single_authentication
bynd_single_authentication.argtypes = [c_void_p, c_void_p, c_char_p, c_char_p]
bynd_single_authentication.restype = c_uint

bynd_management_authentication = lib.bynd_management_authentication
bynd_management_authentication.argtypes = [c_void_p, c_void_p]
bynd_management_authentication.restype = c_uint

# permissions
permissions_get_competition = lib.permissions_get_competition
permissions_get_competition.argtypes = [c_void_p]
permissions_get_competition.restype = c_char_p

permissions_print = lib.permissions_print
permissions_print.argtypes = [c_void_p]

permissions_has_action = lib.permissions_has_action
permissions_has_action.argtypes = [c_void_p, c_char_p]
permissions_has_action.restype = c_bool

# service
auth_service_new = lib.auth_service_new
auth_service_new.restype = c_void_p

auth_service_delete = lib.auth_service_delete
auth_service_delete.argtypes = [c_void_p]

auth_service_create = lib.auth_service_create
auth_service_create.argtypes = [c_char_p, c_char_p]
auth_service_create.restype = c_void_p

# version
bynd_libauth_version_print_full = lib.bynd_libauth_version_print_full
bynd_libauth_version_print_version_id = lib.bynd_libauth_version_print_version_id
bynd_libauth_version_print_version_name = lib.bynd_libauth_version_print_version_name



import datetime
from pymongoose.mongo_types import Types, Schema

class AthleteScore (Schema):
	schema_name = "athletes.scores"

	athlete = None
	heat = None
	score = None

	def __init__ (self, **kwargs):
		self.schema = {
			"workout": {
				"type": Types.ObjectId,
				"ref": "workouts",
				"required": True
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions",
				"required": True
			},
			"heat": {
				"type": Types.ObjectId,
				"ref": "heats",
				"required": True
			},
			"athlete": {
				"type": Types.ObjectId,
				"ref": "athletes",
				"required": True
			},
			"date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			},
			"score": {
				"type": Types.String,
				"required": True
			},
			"tie_break": {
				"type": Types.String,
				"default": "0"
			},
			"withdrawn": {
				"type": Types.Boolean,
				"default": False
			},
			"cap": {
				"type": Types.Boolean,
				"default": False
			},
			"observations": {
				"type": Types.String,
				"default": None
			},
			"last": {
				"type": Types.Boolean,
				"default": True
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f""



from pymongoose.mongo_types import Types, Schema

class AthleteResult (Schema):
	schema_name = "athletes.results"

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions",
				"required": True
			},
			"user": {
				"type": Types.ObjectId,
				"ref": "users",
				"required": True
			},
			"username": {
				"type": Types.String,
				"required": True
			},
			"athlete": {
				"type": Types.ObjectId,
				"ref": "athletes",
				"required": True
			},
			"scores": [
				{
					# reference to each workout
					"workout": {
						"type": Types.ObjectId,
						"ref": "workouts",
						"required": True
					},
					# the actual workout score
					"score": {
						"type": Types.String,
						"required": True
					},
					# the position based on workout score
					"pos": {
						"type": Types.String,
						"required": True
					},
					# how ties are handled
					"tie_break": {
						"type": Types.String,
						"required": True
					},
					# points based on score
					"points": {
						"type": Types.String,
						"required": True
					}
				}
			],
			# the total points from all workouts
			"total": {
				"type": Types.Number,
				"default": 0
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f""



from pymongoose.mongo_types import Types, Schema

REGISTRATION_PHASE_TYPE_NONE = 0
REGISTRATION_PHASE_TYPE_ALL = 1
REGISTRATION_PHASE_TYPE_SELECTED = 2

REGISTRATION_PHASE_STATUS_NONE = 0
REGISTRATION_PHASE_STATUS_PUBLISHED = 1
REGISTRATION_PHASE_STATUS_AVAILABLE = 2
REGISTRATION_PHASE_STATUS_SOLD_OUT = 3
REGISTRATION_PHASE_STATUS_ENDED = 4

class RegistrationPhase (Schema):
	schema_name = "registrations.phases"

	competition = None

	name = None
	description = None

	status = REGISTRATION_PHASE_STATUS_NONE

	price = None
	currency = None

	start_date = None
	end_date = None

	available = None
	sold = None

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": True
			},

			"phase_type": {
				"typer": Types.Number,
				"default": REGISTRATION_PHASE_TYPE_NONE
			},
			"status": {
				"typer": Types.Number,
				"default": REGISTRATION_PHASE_STATUS_NONE
			},

			"price": {
				"type": Types.Number,
				"required": True
			},
			"currency": {
				"type": Types.ObjectId,
				"ref": "currencies",
				"required": True
			},

			"start_date": {
				"type": Types.Date,
				"required": True
			},
			"end_date": {
				"type": Types.Date,
				"required": True
			},

			"available": {
				"type": Types.Number,
				"required": True
			},
			"sold": {
				"typer": Types.Number,
				"default": 0
			},

			"divisions": [
				{
					"type": Types.ObjectId,
					"ref": "divisions"
				}
			]
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"RegistrationPhase: {self.id}"



from pymongoose.mongo_types import Types, Schema

class Heat (Schema):
	schema_name = "heats"

	def __init__ (self, **kwargs):
		self.schema = {
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions"
			},
			"workout": {
				"type": Types.ObjectId,
				"ref": "workouts"
			},
			"division": {
				"type": Types.ObjectId,
				"ref": "divisions"
			},
			"start_time": {
				"type": Types.Date,
				"required": True
			},
			"end_time": {
				"type": Types.Date,
				"required": True
			},
			"athletes": [
				{
					"athlete": {
						"type": Types.ObjectId,
						"ref": "athletes"
					},
					"rail": {
						"type": Types.Number,
						"default": -1
					},
				}
			],
			"teams": [
				{
					"team": {
						"type": Types.ObjectId,
						"ref": "teams"
					},
					"rail": {
						"type": Types.Number,
						"default": -1
					}
				}
			]
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)



import datetime
from pymongoose.mongo_types import Types, Schema

class AthleteFitcard (Schema):
	schema_name = "athletes.fitcards"

	id = None
	athlete = None
	competition = None
	front = None
	back = None

	def __init__ (self, **kwargs):
		self.schema = {
			"athlete": {
				"type": Types.ObjectId,
				"ref": "athletes",
				"required": True
			},
			"competition": {
				"type": Types.ObjectId,
				"ref": "competitions",
				"required": True
			},
			"front": {
				"type": Types.String,
				"required": True
			},
			"back": {
				"type": Types.String,
				"required": True
			},
			"date": {
				"type": Types.Date,
				"default": datetime.datetime.utcnow ()
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Athlete Fitcard: {self.id} - {self.date}"



from pymongoose.mongo_types import Types, Schema

COMPETITION_STATUS_NONE	= 0
COMPETITION_STATUS_PUBLISHED = 1
COMPETITION_STATUS_AVAILABLE = 2
COMPETITION_STATUS_PROGRESS = 3
COMPETITION_STATUS_ENDED = 4

class Competition (Schema):
	schema_name = "competitions"

	id = None
	name = None
	address = None
	description = None
	start_date = None
	end_date = None
	logo = None
	cover = None
	waiver = None

	def __init__ (self, **kwargs):
		self.schema = {
			"status": {
				"type": Types.Number,
				"default": COMPETITION_STATUS_NONE
			},
			"name": {
				"type": Types.String,
				"required": True
			},
			"venue_name": {
				"type": Types.String,
				"required": True
			},
			"address": {
				"text": {
					"type": Types.String,
					"required": True
				},
				"lat": {
					"type": Types.Number,
					"required": True
				},
				"lon": {
					"type": Types.Number,
					"required": True
				}
			},
			"description": {
				"type": Types.String,
				"required": True
			},
			"start_date": {
				"type": Types.Date,
				"required": True
			},
			"end_date": {
				"type": Types.Date,
				"required": True
			},
			"logo": {
				"type": Types.String,
				"required": True
			},
			"cover": {
				"type": Types.String,
				"required": True
			},
			"hashtags": [
				{
					"type": Types.String,
				}
			],
			"url": {
				"type": Types.String
			},
			"social_media": {
				"facebook": {
					"type": Types.String
				},
				"instagram": {
					"type": Types.String
				},
				"twitter": {
					"type": Types.String
				}
			},
			"waiver": {
				"type": Types.String,
				"required": True
			},

			"organizers": [
				{
					"type": Types.ObjectId,
					"ref": "organizers"
				}
			],
			"divisions": [
				{
					"type": Types.ObjectId,
					"ref": "divisions"
				}
			],
			"workouts": [
				{
					"type": Types.ObjectId,
					"ref": "workouts"
				}
			],
			"athletes": [
				{
					"type": Types.ObjectId,
					"ref": "athletes"
				}
			]

		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"Competition: {self.name} - {self.id}"



from pymongoose.mongo_types import Types, Schema

class PermissionsAction (Schema):
	schema_name = "permissions.actions"

	def __init__ (self, **kwargs):
		self.schema = {
			"name": {
				"type": Types.String,
				"required": True
			},
			"description": {
				"type": Types.String,
				"required": True
			}
		}

		super ().__init__ (self.schema_name, self.schema, kwargs)

	def __str__ (self):
		return f"PermissionsAction: {self.id} - {self.name}"
